--By JupiterXLI

renoise.tool():add_menu_entry {
  name = "DSP Device:Randomise Parameters...",
  invoke = function () dialogue(1) end
}
renoise.tool():add_menu_entry {
  name = "Instrument Box:Randomise Parameters...",
  invoke = function () dialogue(2) end
}
renoise.tool():add_keybinding {
  name = "Global:Tools:Randomise Parameters (Device)",
  invoke = function() dialogue(1) end
}
renoise.tool():add_keybinding {
  name = "Global:Tools:Randomise Parameters (Instrument)",
  invoke = function() dialogue(2) end
}


-------------VARIABLES-------------------
local sel_track = nil
local sel_device = nil
local trk_dev = nil
local sel_inst = nil
local inst_plug = nil
local preset_dialog = nil
--------------------------------------

--------------FUNCTIONS----------------  
function get_indices(x)

  local index_list = {}
  if x == "Device" then
    sel_track = renoise.song().selected_track_index
    sel_device = renoise.song().selected_device_index
    trk_dev = renoise.song():track(sel_track):device(sel_device)
    
    for x = 1, #trk_dev.parameters do 
      local index_name = ("%s %s"):format(x,trk_dev:parameter(x).name)
      table.insert(index_list,index_name)
    end    
  else
    --get inst variables
    sel_inst = renoise.song().selected_instrument_index
    inst_plug = renoise.song():instrument(sel_inst).plugin_properties.plugin_device.parameters
  
    for x = 1, #inst_plug do
      local index_name = ("%s %s"):format(x,inst_plug[x].name)
      table.insert(index_list,index_name)
    end
  end
  return index_list
end


function plugin_rand(list,min,max,x,edit_choice)  

  --setup table
  local param_tab = {}
  if x == "Device" then
    for i=1,#trk_dev.parameters do
      table.insert(param_tab,i)
    end
  else
    for i=1,#inst_plug do
      table.insert(param_tab,i)
    end
  end
  
    --check for list and edit choice
  if #list ~= 0 then --if retain or edit then redo table
    if edit_choice == 2 then --if edit then replace table
      param_tab = {}
      for _,v in pairs(list) do
        table.insert(param_tab,v)
      end
    else --retain removes those items from table
      for k,v in pairs(param_tab) do
        for _,i in pairs(list) do
          if v == i then
            param_tab[k] = nil
          end
        end
      end
    end
  end
  
  --iterate through list and assign random value to each parameter
  if x == "Device" then
    for _,i in pairs(param_tab) do
      local rand = math.random(min,max)*0.01 -- turn to float
      --min or max is not default, recalculate scale = * scaledif + scalemin
      if trk_dev:parameter(i).value_min ~= 0.0 or trk_dev:parameter(i).value_max ~= 1.0 then 
        local scalemin = trk_dev:parameter(i).value_min
        local scaledif = trk_dev:parameter(i).value_max - scalemin
        
        rand = (rand * scaledif) + scalemin -- 9.9999999747524e-007 Flanger Rate
      end
      trk_dev:parameter(i).value = rand
    end
  else
    for _,i in pairs(param_tab) do 
      local rand = math.random(min,max)*0.01
      if inst_plug[i].value_min ~= 0.0 or inst_plug[i].value_max ~= 1.0 then         
        local scalemin = inst_plug[i].value_min
        local scaledif = inst_plug[i].value_max - scalemin

        rand = (rand * scaledif) + scalemin --rand and floats        
      end
      
      inst_plug[i].value = rand
    end
  end
  
end

----------------------------GUI------------------------------

--------------PRESET LOADER-----------------------------------
function preset_window()
  local vb= renoise.ViewBuilder()
  
  local filename = "jxli_randomise_presets.txt"
  
  local t = {}
  for line in io.lines(filename,'r') do
    table.insert(t,line)
  end
  
  local combline=table.concat(t,"\n")
  
  local presets = vb:horizontal_aligner{
      vb:multiline_textfield{
      width = 300,
      height = 200,
      id="preset_field",
      text=""..combline,
      font="mono"
      }
  }
  
  local content = vb:vertical_aligner {  
    vb:horizontal_aligner{
      mode = "justify",
      vb:text {
        text="Edit for storing presets."
      },
      vb:button {
        text="Save",
        notifier = function()
          local f = assert(io.open(filename, 'w'))
          f:write(vb.views.preset_field.text)
          f:close()
          
          t = {}
          for line in io.lines(filename,'r') do
            table.insert(t,line)
          end
          
          combline = table.concat(t,"\n")
          
          vb.views.preset_field.text = combline
        end
      }
    },
    presets
  }
  preset_dialog = renoise.app():show_custom_dialog("Preset access",content)
end

function dialogue(def_selection) --def_selection refers to index of cho_list
  local vb = renoise.ViewBuilder()
  local cho_list = {"Device","Instrument"}
  local max_index = 256 --test number meaningless
  local def_choice = cho_list[def_selection]--make selection in list
  
  local res_field = vb:row{
      vb:multiline_text{
		  height = 150,
		  width = 400,
        id="index_list",
        font="mono",
        text=""
      }
  }
  
  local function get_params() --gets parameters and prints them out
    local indices = ""

    --error checks
    sel_device = renoise.song().selected_device_index 
    sel_inst = renoise.song().selected_instrument_index
    inst_plug = renoise.song():instrument(sel_inst).plugin_properties.plugin_device

    if vb.views.index_choice.value == 1 and sel_device == 0 then          
      indices = {"Error","Select a device!!"}
    elseif vb.views.index_choice.value == 2 and inst_plug == nil then
      indices = {"Error","Select a plugin instrument!!"}
	 else --get parameters
      indices =get_indices(cho_list[vb.views.index_choice.value])
    end
	 
	 --supposedly faster method
	 local t = {}
    for _,v in pairs(indices) do
      table.insert(t,v)
    end
	
	 vb.views.index_list.text = table.concat(t,"\n") --print them out

    max_index = #indices --for range error check later
  end
  
  local content = vb:vertical_aligner {
    margin = renoise.ViewBuilder.DEFAULT_DIALOG_MARGIN,
    vb:horizontal_aligner{
      vb:text{text="First select Device or Instrument.\nChoose whether to retain or edit the indices which you can input. Input the\nindex(number) of any parameters you want or leave empty and execute. Have fun.\nIf you want to change to another device/instrument first select that object\nin Renoise, change the selection in the drop-down menu and then\nchange back, this will reset the tool.\nFor a range of indices use '-', ex. 100 - 112.\n\n"}
    },
        
    vb:horizontal_aligner{
    mode = "justify",
      vb:popup{
        id = "index_choice",
        items=cho_list,
        value=def_selection,
        notifier=function()
			 get_params()
        end
      },
      vb:switch{
        width = 100,
        id = "keep_edit_choice",
        items={"Retain","Edit"},
        value=1
      }
    },
    
    vb:horizontal_aligner{
    mode = "left",
      vb:text{text="Range:  "},
      vb:text{text="Min "},
      vb:valuebox{
        id="min_range_choice",
        min=0,
        max=99,
        value=0
      },
      vb:text{text="Max "},
      vb:valuebox{
        id="max_range_choice",
        min=1,
        max=100,
        value=100
      },
    },
    
    vb:horizontal_aligner {
    mode = "justify",
      vb:text{text="Selected Indices:"},
      vb:textfield{
        width = 200,
        id="user_table",
        value=""
      },
      vb:button {
        text = "Execute",   
        notifier = function()
          local strin = vb.views.user_table.value
          local user_tab = {}
			 for item in strin:gmatch("%d+") do
            local num_it = tonumber(item)
            if num_it <= max_index then
              table.insert(user_tab,num_it)
            end
          end
          
          --range GUN
          for range_start in strin:gmatch("(%d+)%s*%-%s*%d+") do
				local range_st = tonumber(range_start)
            for range_end in strin:gmatch("%d+%s*%-%s*(%d+)") do
				  local range_nd = tonumber(range_end)
              if range_nd > range_st then
                for x = range_st,range_nd do
                  if x <= max_index then -- out of upr range just ends at max value
                    table.insert(user_tab,x)
                  end
                end
              end
            end
          end
          
          local min = vb.views.min_range_choice.value
          local max = vb.views.max_range_choice.value
            
          if min > max then
            max = vb.views.min_range_choice.value
            min = vb.views.max_range_choice.value
          elseif min == max then
            max = 100
            min = 0
          end
          
          local inst_dev = cho_list[vb.views.index_choice.value]
          local edit_choice = vb.views.keep_edit_choice.value
          
          plugin_rand(user_tab,min,max,inst_dev,edit_choice)
        end
      },
    vb:button {
      text="preset load",
      notifier = function()
		  if not (preset_dialog and preset_dialog.visible) then
		    preset_window()
			 preset_dialog:show()
		  end
      end
    },
    },
    res_field
  }
  
  get_params() --for default settings
  
  renoise.app():show_custom_dialog("JupiterXLI Randomise Parameters",content)
end
